/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada. 
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;

/**
 * Dialog to allow a user to rename a node
 * @author Chris Bennett
 *
 */
public class NodeRenameDialog extends JDialog {
	
	private boolean cancelled = false;
	private boolean restored = false;
	private boolean changed = false;
	private final static String DEFAULT_TITLE = "Rename Node";
	
	public NodeRenameDialog(final ShrimpNode node) {
		this(node, DEFAULT_TITLE);
	}

	public NodeRenameDialog(final ShrimpNode node, String title) {
		super(ApplicationAccessor.getApplication().getParentFrame(), title, true);

		JLabel jLabel1 = new JLabel("Name: ");
		jLabel1.setPreferredSize(new Dimension(50, 20));
		jLabel1.setHorizontalAlignment(javax.swing.JLabel.RIGHT);

		final JTextField tfldName = new JTextField();
		tfldName.setPreferredSize(new Dimension(120, 20));
		tfldName.setText("" + node.getName());
		
		JPanel jPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		jPanel.add(jLabel1);
		jPanel.add(tfldName);

		JButton okBtn = new JButton(new AbstractAction("OK") {
			public void actionPerformed(ActionEvent e) {
				node.rename(tfldName.getText());
				changed = true;
				dispose();
			}
		});

		JButton restoreBtn = new JButton(new AbstractAction("Restore") {
			public void actionPerformed(ActionEvent e) {
				node.restoreName();
				restored = true;
				dispose();
			}
		});
		restoreBtn.setEnabled(node.getSavedNames().length > 0);

		JButton cancelBtn = new JButton(new AbstractAction("Cancel") {
			public void actionPerformed(ActionEvent e) {
				cancelled = true;
				dispose();
			}
		});
		
		JPanel buttonsPanel = new JPanel(new FlowLayout());
		buttonsPanel.add(okBtn);
		buttonsPanel.add(restoreBtn);
		buttonsPanel.add(cancelBtn);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(jPanel, BorderLayout.CENTER);
		getContentPane().add(buttonsPanel, BorderLayout.SOUTH);
		pack();

		// put in center of screen
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (int) (screen.getWidth() - this.getWidth()) / 2;
		int y = (int) (screen.getHeight() - this.getHeight()) / 2;
		setLocation(x, y);
		setSize(250, 120);

		getRootPane().setDefaultButton(okBtn);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				cancelled = true;
				dispose();
			}
		});
		setVisible(true);
	}

	/**
	 * Returns true if user has pushed cancel button.
	 */
	public boolean isCancelled () {
		return cancelled;
	}

	/**
	 * Returns true if user has pushed restore button.
	 */
	public boolean isRestored () {
		return restored;
	}

	/**
	 * Returns true if user has pushed ok button.
	 */
	public boolean isChanged() {
		return changed;
	}
}
